
/*

- Temperature and humidity on display with AM2302
- Showing actual, min and max values (C, %)

PIC:  16F886
Osc:  8.00 MHz, internal osc.
Disp: GLCD 128x64 pixel KS108 Controller

V0.1      Sept 2013   construction
V1.0      Feb  2014   clean-up
V1.0.0.0  Feb  2014   release for Libstock
V1.0.0.1  Feb  2014   bugfix fConvHumToStr instead of fConvTempToStr in main.c
V1.1.0.0  Sept 2014   Sensor name AM2303 instead of AM2302 corrected
V2.0.0.0  Sept 2014   checksum test added, some small changes and improvements

Software tested with AM2302 which has a DHT22 implemented.

- Sensor connected to PORTA1 --> see header.h
  pull-up sensor data pin with 10kOhm
- GLCD connections --> check code

*/


#include "header.h"


/* Glcd module connections */
char GLCD_DataPort at PORTC;

sbit GLCD_CS2 at RB0_bit;
sbit GLCD_CS1 at RB1_bit;
sbit GLCD_RS  at RB2_bit;
sbit GLCD_RW  at RB3_bit;
sbit GLCD_EN  at RB4_bit;
sbit GLCD_RST at RB5_bit;

sbit GLCD_CS2_Direction at TRISB0_bit;
sbit GLCD_CS1_Direction at TRISB1_bit;
sbit GLCD_RS_Direction  at TRISB2_bit;
sbit GLCD_RW_Direction  at TRISB3_bit;
sbit GLCD_EN_Direction  at TRISB4_bit;
sbit GLCD_RST_Direction at TRISB5_bit;


/* Global variables */
unsigned int humidWord, tempWord, humidOld, tempOld;
unsigned int humidMin, humidMax, tempMin, tempMax;



void main(){
  fInit();

  while(1){
    fSendRequest();

    if(fCheckSumTest() == 1){  // update data only if check sum is correct

       /* update temp */
       if((tempOld - tempWord) != 0){
          Glcd_Rectangle_Round_Edges_Fill(0, 0, 63, 30, 1, 0);
          Glcd_Write_Text_Adv(fConvTempToStr(tempWord), 0, 0);
          tempOld  = tempWord;

          if(tempWord < tempMin){
            tempMin = tempWord;
            Glcd_Write_Text(fConvTempToStr(tempMin), 105,2,1);
          }
          else if (tempWord > tempMax){
            tempMax = tempWord;
            Glcd_Write_Text(fConvTempToStr(tempMax), 105,0,1);
          }
       }
   
       /* update humidity */
       if((humidOld - humidWord) != 0){
          Glcd_Rectangle_Round_Edges_Fill(0, 31, 69, 64, 1, 0);
          Glcd_Write_Text_Adv(fConvHumToStr(humidWord), 0, 33);
          humidOld = humidWord;

          if(humidWord < humidMin){
            humidMin = humidWord;
            Glcd_Write_Text(fConvHumToStr(humidMin), 105,7,1);
          }
          else if (humidWord > humidMax){
            humidMax = humidWord;
            Glcd_Write_Text(fConvHumToStr(humidMax), 105,5,1);
          }
       }
    } /* if checksum */
    else {
       Glcd_Write_Text("sensor error", 1,6,1);
    }
    
    fDelay();
    } /* while(1) */
}